#!/bin/sh
#
#	Create two executables from .c files in the local directory,
#	and two in a subdirectory.  Remove one of the local
#	executables and .o files using -r; check that it worked
#	correctly.  Remove the executables and .o files from the
#	subdirectory using -r; check that it worked correctly.
#	Rebuild the world.  Remove the world using -r; check that
#	it worked correctly.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-r]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	if test X$PRESERVE = X -a X$1 = X ; then
		cd $here
		find $work -type d -user $USER -exec chmod u+w {} \;
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir subdir
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >Construct
$env = new cons;
Program $env 'a', 'a.c';
Program $env 'b', 'b.c';
Build 'subdir/Conscript';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >a.c
main()
{
	printf("a.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >b.c
main()
{
	printf("b.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/Conscript
$env = new cons;
Program $env 'c', 'c.c';
Program $env 'd', 'd.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/c.c
main()
{
	printf("subdir/c.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/d.c
main()
{
	printf("subdir/d.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
./a >a.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - a.1.out <<'_EOF_'
a.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
./b >b.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - b.1.out <<'_EOF_'
b.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
subdir/c >c.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - c.1.out <<'_EOF_'
subdir/c.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
subdir/d >d.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - d.1.out <<'_EOF_'
subdir/d.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

$prog . >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - cons.2.out <<'_EOF_'
cons: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

$prog -r b.o b >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
./a >a.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - a.3.out <<'_EOF_'
a.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
if test -e b.o ; then fail 3 ; fi
if test -e b ; then fail 3 ; fi
subdir/c >c.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - c.3.out <<'_EOF_'
subdir/c.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
subdir/d >d.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - d.3.out <<'_EOF_'
subdir/d.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

$prog -r subdir >cons.4.out 2>cons.4.err
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
./a >a.4.out
if test $? -ne 0 ; then fail 4 ; fi
diff - a.4.out <<'_EOF_'
a.c
_EOF_
if test $? -ne 0 ; then fail 4 ; fi
if test -e b.o ; then fail 4 ; fi
if test -e b ; then fail 4 ; fi
if test -e subdir/c.o ; then fail 4 ; fi
if test -e subdir/c ; then fail 4 ; fi
if test -e subdir/d.o ; then fail 4 ; fi
if test -e subdir/d ; then fail 4 ; fi

$prog . >cons.5.out 2>cons.5.err
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
subdir/c >c.5.out
if test $? -ne 0 ; then fail 5 ; fi
diff - c.5.out <<'_EOF_'
subdir/c.c
_EOF_
if test $? -ne 0 ; then fail 5 ; fi
subdir/d >d.5.out
if test $? -ne 0 ; then fail 5 ; fi
diff - d.5.out <<'_EOF_'
subdir/d.c
_EOF_
if test $? -ne 0 ; then fail 5 ; fi

$prog -r . >cons.6.out 2>cons.6.err
if test $? -ne 0 ; then fail 6 cons.6.out ; fi
if test -e a.o ; then fail 6 ; fi
if test -e a ; then fail 6 ; fi
if test -e b.o ; then fail 6 ; fi
if test -e b ; then fail 6 ; fi
if test -e subdir/c.o ; then fail 6 ; fi
if test -e subdir/c ; then fail 6 ; fi
if test -e subdir/d.o ; then fail 6 ; fi
if test -e subdir/d ; then fail 6 ; fi

#
pass
